import(stats)
import(graphics)
importFrom("grDevices", "col2rgb", "palette", "rgb")
importFrom("utils", "type.convert")
export(ca, iterate.mjca, mjca, pchlist, plot3d.ca, plot.ca, plot.mjca, cacoord, caconv, multilines)

S3method(plot, ca)
S3method(plot, mjca)
S3method(print, ca)
S3method(print, mjca)
S3method(print, summary.ca)
S3method(print, summary.mjca)
S3method(summary, ca)
S3method(summary, mjca)

S3method(ca, matrix)
S3method(ca, data.frame)
S3method(ca, formula)
S3method(ca, table)
S3method(ca, xtabs)

S3method(mjca, table)
S3method(mjca, array)
S3method(mjca, data.frame)

if(getRversion() >= "3.6.0") {
  S3method(rgl::plot3d, ca)
  }